' Fig. 8.13: GradeBook.vb
' GradeBook class uses an array to store test grades.
Public Class GradeBook
   Private courseNameValue As String ' name of course 
   Private grades As Integer() ' array of student grades             

   ' two-argument constructor initializes courseNameValue and grades
   Public Sub New(ByVal name As String, ByVal gradesArray As Integer())
      CourseName = name ' initializes courseNameValue via property
      grades = gradesArray ' store reference to gradesArray
   End Sub ' New

   ' property CourseName
   Public Property CourseName() As String
      Get
         Return courseNameValue
      End Get

      Set(ByVal name As String)
         courseNameValue = name
      End Set
   End Property ' Course Name

   ' display a welcome message to the GradeBook user
   Public Sub DisplayMessage()
      Console.WriteLine("Welcome to the grade book for " & vbCrLf & _
         CourseName & vbCrLf)
   End Sub ' DisplayMessage

   ' perform various operations on the data
   Public Sub ProcessGrades()
      OutputGrades() ' output grades array

      ' call method GetAverage to calculate the average grade
      Console.WriteLine("Class average is {0:F2}", GetAverage())

      ' call methods GetMinimum and GetMaximum
      Console.WriteLine("Lowest grade is {0}", GetMinimum())
      Console.WriteLine("Highest grade is {0}", GetMaximum())

      ' call OutputBarChart to print grade distribution chart
      OutputBarChart()
   End Sub ' ProcessGrades

   ' find minimum grade
   Public Function GetMinimum() As Integer
      Dim lowGrade As Integer = grades(0) ' assume grades(0) is smallest

      ' loop through grades array
      For Each grade As Integer In grades
         ' if grade lower than lowGrade, assign it to lowGrade
         If grade < lowGrade Then
            lowGrade = grade ' new lowest grade
         End If
      Next

      Return lowGrade ' return lowest grade
   End Function ' GetMinimum

   ' find maximum grade
   Public Function GetMaximum() As Integer
      Dim highGrade As Integer = grades(0) ' assume grades(0) is largest

      ' loop through grades array
      For Each grade As Integer In grades
         ' if grade greater than highGrade, assign it to highGrade
         If grade > highGrade Then
            highGrade = grade ' new highest grade
         End If
      Next

      Return highGrade ' return highest grade
   End Function ' GetMaximum

   ' determine average grade for test
   Public Function GetAverage() As Double
      Dim total As Integer = 0 ' initialize total

      ' sum grades for one student
      For Each grade As Integer In grades
         total += grade
      Next

      ' return average of grades
      Return (total / grades.Length)
   End Function ' GetAverage

   ' output bar chart displaying grade distribution
   Public Sub OutputBarChart()
      Console.WriteLine(vbCrLf & "Grade distribution:")

      ' stores frequency of grades in each range of 10 grades
      Dim frequency As Integer() = New Integer(10) {}

      ' for each grade, increment the appropriate frequency
      For Each grade As Integer In grades
         frequency(grade \ 10) += 1
      Next

      ' for each grade frequency, print bar in chart
      For count As Integer = 0 To frequency.GetUpperBound(0)
         ' output bar label ( "00-09: ", ..., "90-99: ", "100: " )
         If count = 10 Then
            Console.Write("{0, 5:D}: ", 100)
         Else
            Console.Write("{0, 2:D2}-{1, 2:D2}: ", _
               count * 10, count * 10 + 9)
         End If

         ' print bar of asterisks
         For stars As Integer = 0 To frequency(count) - 1
            Console.Write("*")
         Next

         Console.WriteLine() ' start a new line of output
      Next
   End Sub ' OutputBarChart

   ' output the contents of the grades array
   Public Sub OutputGrades()
      Console.WriteLine("The grades are:" & vbCrLf)

      ' output each student's grade
      For student As Integer = 0 To grades.GetUpperBound(0)
         Console.WriteLine("Student {0, 2:D}: {1, 3:D}", _
            student + 1, grades(student))
      Next

      Console.WriteLine()
   End Sub ' OutputGrades
End Class ' GradeBook

' **************************************************************************
' * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************
