' Fig. 8.17: GradeBook.vb
' Grade book using a rectangular array to store grades.
Public Class GradeBook
   Private courseNameValue As String ' name of course 
   Private grades As Integer(,) ' rectangular array of student grades             

   ' two-argument constructor initializes courseNameValue and Grades
   Public Sub New(ByVal name As String, ByVal gradesArray As Integer(,))
      CourseName = name ' initializes courseNameValue via property
      grades = gradesArray ' store grades
   End Sub ' New

   ' property CourseName
   Public Property CourseName() As String
      Get
         Return courseNameValue
      End Get

      Set(ByVal name As String)
         courseNameValue = name
      End Set
   End Property ' Course Name

   ' display a welcome message to the GradeBook user
   Public Sub DisplayMessage()
      Console.WriteLine("Welcome to the grade book for " & vbCrLf & _
         CourseName & vbCrLf)
   End Sub ' DisplayMessage

   ' perform various operations on the data
   Public Sub ProcessGrades()
      OutputGrades() ' output grades array

      ' call methods GetMinimum and GetMaximum
      Console.WriteLine("Lowest grade in the grade book is {0}", _
         GetMinimum())
      Console.WriteLine("Highest grade in the grade book is {0}", _
         GetMaximum())

      ' call OutputBarChart to print grade distribution chart
      OutputBarChart()
   End Sub ' ProcessGrades

   ' find minimum grade
   Public Function GetMinimum() As Integer
      ' assume first element of grades array is smallest
      Dim lowGrade As Integer = grades(0, 0)

      ' loop through grades array
      For i As Integer = 0 To grades.GetUpperBound(0)
         ' loop through columns of current row
         For j As Integer = 0 To grades.GetUpperBound(1)
            ' if grade lower than lowGrade, assign it to lowGrade
            If grades(i, j) < lowGrade Then
               lowGrade = grades(i, j) ' new lowest grade
            End If
         Next
      Next

      Return lowGrade ' return lowest grade
   End Function ' GetMinimum

   ' find maximum grade
   Public Function GetMaximum() As Integer
      ' assume first element of grades array is largest
      Dim highGrade As Integer = grades(0, 0)

      ' loop through grades array
      For i As Integer = 0 To grades.GetUpperBound(0)
         ' loop through columns of current row
         For j As Integer = 0 To grades.GetUpperBound(1)
            ' if grade greater than highGrade, assign it to highGrade
            If grades(i, j) > highGrade Then
               highGrade = grades(i, j) ' new highest grade
            End If
         Next
      Next

      Return highGrade ' return highest grade
   End Function ' GetMaximum

   ' determine average grade for particular student's grades
   Public Function GetAverage(ByVal row As Integer) As Double
      Dim total As Integer = 0 ' initialize total

      ' sum grades for one student            
      For column As Integer = 0 To grades.GetUpperBound(1)
         total += grades(row, column)
      Next

      ' return average of grades
      Return (total / (grades.GetUpperBound(1) + 1))
   End Function ' GetAverage

   ' output bar chart displaying grade distribution
   Public Sub OutputBarChart()
      Console.WriteLine(vbCrLf & "Overall grade distribution:")

      ' stores frequency of grades in each range of 10 grades
      Dim frequency As Integer() = New Integer(10) {}

      ' for each grade, increment the appropriate frequency
      For i As Integer = 0 To grades.GetUpperBound(0)
         For j As Integer = 0 To grades.GetUpperBound(1)
            frequency(grades(i, j) \ 10) += 1
         Next
      Next

      ' for each grade frequency, print bar in chart
      For count As Integer = 0 To frequency.GetUpperBound(0)
         ' output bar label ( "00-09: ", ..., "90-99: ", "100: " )
         If count = 10 Then
            Console.Write("{0, 5:D}: ", 100)
         Else
            Console.Write("{0, 2:D2}-{1, 2:D2}: ", _
               count * 10, count * 10 + 9)
         End If

         ' print bar of asterisks
         For stars As Integer = 0 To frequency(count) - 1
            Console.Write("*")
         Next

         Console.WriteLine() ' start a new line of output
      Next
   End Sub ' OutputBarChart

   ' output the contents of the grades array
   Public Sub OutputGrades()
      Console.WriteLine("The grades are:" & vbCrLf)
      Console.Write("            ") ' align column heads

      ' create a column heading for each of the tests
      For test As Integer = 0 To grades.GetUpperBound(1)
         Console.Write("Test {0:D}  ", test + 1)
      Next

      Console.WriteLine("Average") ' student average column heading

      ' create rows/columns of text representing array grades
      For student As Integer = 0 To grades.GetUpperBound(0)
         Console.Write("Student {0, 2:D}", student + 1)

         ' output student's grades
         For counter As Integer = 0 To grades.GetUpperBound(1)
            Console.Write("{0, 8:D}", grades(student, counter))
         Next

         ' call method GetAverage to calculate student's average grade;
         ' pass row of grades as the argument to GetAverage
         Dim average As Double = GetAverage(student)
         Console.WriteLine("{0, 9:F2}", average)
      Next

      Console.WriteLine()
   End Sub ' OutputGrades
End Class ' GradeBook

' **************************************************************************
' * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************
