' Fig. 9.1: Time.vb
' Represents time to clients in 12-hour and 24-hour clock formats.
Class Time
   Inherits Object ' if omitted, Object is inherited implicitly

   ' declare Integer instance variables for the hour, minute and second
   Private hourValue As Integer ' 0 - 23
   Private minuteValue As Integer ' 0 - 59
   Private secondValue As Integer ' 0 - 59

   ' Time constructor initializes instance variables 
   ' when a Time object is created                  
   Public Sub New()
      SetTime(12, 0, 0) ' initialize hour to noon; minute, second to 0
   End Sub ' New                                                       

   ' set a new time value using universal time, check validity of the
   ' data, set invalid hour to noon, set invalid minute, second to zero
   Public Sub SetTime(ByVal hh As Integer, _
      ByVal mm As Integer, ByVal ss As Integer)
      Hour = hh ' set hourValue using Hour property    
      Minute = mm ' set minuteValue using Minute property
      Second = ss ' set secondValue using Second property
   End Sub ' SetTime

   ' property Hour                                              
   Public Property Hour() As Integer
      Get ' return hourValue                                    
         Return hourValue
      End Get

      Set(ByVal value As Integer) ' set hourValue
         If (value >= 0 AndAlso value < 24) Then ' in range 0-23
            hourValue = value ' value is valid
         Else ' value is invalid
            hourValue = 12 ' set to default of noon
         End If
      End Set
   End Property ' Hour

   ' property Minute                                            
   Public Property Minute() As Integer
      Get ' return minuteValue                                  
         Return minuteValue
      End Get

      Set(ByVal value As Integer) ' set minuteValue
         If (value >= 0 AndAlso value < 60) Then ' in range 0-59
            minuteValue = value ' value is valid
         Else ' value is invalid
            minuteValue = 0 ' set to default of 0
         End If
      End Set
   End Property ' Minute                                        

   ' property Second                                            
   Public Property Second() As Integer
      Get ' return secondValue                                  
         Return secondValue
      End Get

      Set(ByVal value As Integer) ' set secondValue
         If (value >= 0 AndAlso value < 60) Then ' in range 0-59
            secondValue = value ' value is valid
         Else ' value is invalid
            secondValue = 0 ' set to default of 0
         End If
      End Set
   End Property ' Second                                        

   ' convert Time to a String in universal-time (24-hour clock) format
   Public Function ToUniversalString() As String
      Return String.Format("{0}:{1:D2}:{2:D2}", Hour, Minute, Second)
   End Function ' ToUniversalString

   ' convert Time to a String in standard-time (12-hour clock) format
   Public Overrides Function ToString() As String
      Dim suffix As String ' AM or PM suffix
      Dim standardHour As Integer ' a standard hour in the range 1-12

      ' determine whether the 12-hour clock suffix should be AM or PM
      If Hour < 12 Then
         suffix = "AM" ' note space preceding AM
      Else
         suffix = "PM" ' note space preceding PM
      End If

      ' convert hour from universal-time format to standard-time format
      If (Hour = 12 OrElse Hour = 0) Then
         standardHour = 12
      Else
         standardHour = Hour Mod 12 ' 1 through 11, AM or PM
      End If

      Return String.Format("{0}:{1:D2}:{2:D2} {3}", _
         standardHour, Minute, Second, suffix)
   End Function ' ToString
End Class ' Time


' **************************************************************************
' * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************