' Fig. 17.26: FrmChessGame.vb
' Chess Game graphics code.
Public Class FrmChessGame
   Private chessTile As New ArrayList() ' for tile images
   Private chessPieces As New ArrayList() ' for chess pieces
   Private selectedIndex As Integer = -1 ' index for selected piece
   Private board(,) As Integer = New Integer(7, 7) {} ' board array
   Private Const TILESIZE As Integer = 75 ' chess tile size in pixels

   ' load tile bitmaps and reset game
   Private Sub FrmChessGame_Load(ByVal sender As Object, _
      ByVal e As System.EventArgs) Handles Me.Load
      ' load chess board tiles
      chessTile.Add(Bitmap.FromFile("images\lightTile1.png"))
      chessTile.Add(Bitmap.FromFile("images\lightTile2.png"))
      chessTile.Add(Bitmap.FromFile("images\darkTile1.png"))
      chessTile.Add(Bitmap.FromFile("images\darkTile2.png"))

      ResetBoard() ' initialize board
      Invalidate() ' refresh form
   End Sub ' FrmChessGame_Load

   ' initialize pieces to start and rebuild board
   Private Sub ResetBoard()
      Dim current As Integer = -1
      Dim piece As ChessPiece
      Dim random As New Random()
      Dim light As Boolean = False
      Dim type As Integer

      chessPieces.Clear() ' ensure empty arraylist

      ' load whitepieces image
      Dim whitePieces As Bitmap = _
         CType(Image.FromFile("images\whitePieces.png"), Bitmap)

      ' load blackpieces image
      Dim blackPieces As Bitmap = _
         CType(Image.FromFile("images\blackPieces.png"), Bitmap)

      ' set whitepieces to be drawn first
      Dim selected As Bitmap = whitePieces

      ' traverse board rows in outer loop
      For row As Integer = 0 To board.GetUpperBound(0)
         ' if at bottom rows, set to black pieces images
         If row > 5 Then
            selected = blackPieces
         End If

         ' traverse board columns in inner loop
         For column As Integer = 0 To board.GetUpperBound(1)
            ' if first or last row, organize pieces
            If row = 0 OrElse row = 7 Then
               Select Case column
                  Case 0, 7 ' set current piece to rook
                     current = ChessPiece.Types.ROOK
                  Case 1, 6 ' set current piece to knight
                     current = ChessPiece.Types.KNIGHT
                  Case 2, 5 ' set current piece to bishop
                     current = ChessPiece.Types.BISHOP
                  Case 3 ' set current piece to king
                     current = ChessPiece.Types.KING
                  Case 4 ' set current piece to queen
                     current = ChessPiece.Types.QUEEN
               End Select

               ' create current piece at start position
               piece = New ChessPiece(current, _
                  column * TILESIZE, row * TILESIZE, selected)

               chessPieces.Add(piece) ' add piece to arraylist
            End If

            ' if second or seventh row, organize pawns
            If row = 1 OrElse row = 6 Then
               piece = New ChessPiece(ChessPiece.Types.PAWN, _
                  column * TILESIZE, row * TILESIZE, selected)
               chessPieces.Add(piece) ' add piece to arraylist
            End If

            type = random.Next(0, 2) ' determine board piece type

            If light Then ' set light tile
               board(row, column) = type
               light = False
            Else ' set dark tile
               board(row, column) = type + 2
               light = True
            End If
         Next column

         light = Not light ' account for new row tile color switch
      Next row
   End Sub ' ResetBoard

   ' display board in form OnPaint event
   Private Sub FrmChessGame_Paint(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
      Dim graphicsObject As Graphics = e.Graphics ' obtain graphics object
      graphicsObject.TranslateTransform(0, 24) ' adjust origin

      For row As Integer = 0 To board.GetUpperBound(0)
         For column As Integer = 0 To board.GetUpperBound(1)
            ' draw image specified in board array
            graphicsObject.DrawImage( _
               CType(chessTile(board(row, column)), Image), _
               New Point(TILESIZE * column, (TILESIZE * row)))
         Next column
      Next row
   End Sub ' FrmChessGame_Paint

   ' return index of piece that intersects point
   ' optionally exclude a value
   Private Function CheckBounds(ByVal pointValue As Point, _
      ByVal exclude As Integer) As Integer
      Dim rectangleValue As Rectangle ' current bounding rectangle

      For i As Integer = 0 To chessPieces.Count - 1
         ' get piece rectangle
         rectangleValue = GetPiece(i).GetBounds()

         ' check if rectangle contains point
         If rectangleValue.Contains(pointValue) And i <> exclude Then
            Return i
         End If
      next 

      Return -1
   End Function ' CheckBounds

   ' handle picBoard Paint event
   Private Sub picBoard_Paint(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.PaintEventArgs) _
      Handles picBoard.Paint
      ' draw all pieces
      For i As Integer = 0 To chessPieces.Count - 1
         GetPiece(i).Draw(e.Graphics)
      Next
   End Sub ' picBoard_Paint

   ' handle picBoard MouseDown event
   Private Sub picBoard_MouseDown(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.MouseEventArgs) _
      Handles picBoard.MouseDown
      ' determine selected piece
      selectedIndex = CheckBounds(New Point(e.X, e.Y), -1)
   End Sub ' picBoard_MouseDown

   ' if piece is selected, move it
   Private Sub picBoard_MouseMove(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.MouseEventArgs) _
      Handles picBoard.MouseMove

      If selectedIndex > -1 Then
         Dim region As New Rectangle(e.X - TILESIZE * 2, _
            e.Y - TILESIZE * 2, TILESIZE * 4, TILESIZE * 4)

         ' set piece center to mouse
         GetPiece(selectedIndex).SetLocation( _
            e.X - TILESIZE \ 2, e.Y - TILESIZE \ 2)

         picBoard.Invalidate(region) ' refresh region
      End If
   End Sub ' picBoard_MouseMove

   ' on mouse up deselect piece and remove taken piece
   Private Sub picBoard_MouseUp(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.MouseEventArgs) _
      Handles picBoard.MouseUp

      Dim remove As Integer = -1

      ' if chess piece was selected
      If selectedIndex > -1 Then
         Dim current As New Point(e.X, e.Y)
         Dim newPoint As New Point( _
            current.X - (current.X Mod TILESIZE), _
            current.Y - (current.Y Mod TILESIZE))

         ' check bounds with point, exclude selected piece
         remove = CheckBounds(current, selectedIndex)

         ' snap piece into center of closest square
         GetPiece(selectedIndex).SetLocation(newPoint.X, newPoint.Y)
         selectedIndex = -1 ' deselect piece

         ' remove taken piece
         If remove > -1 Then
            chessPieces.RemoveAt(remove)
         End If
      End If

      picBoard.Invalidate() ' ensure artifact removal   
   End Sub ' picBoard_MouseUp

   ' helper function to convert 
   ' ArrayList object to ChessPiece
   Private Function GetPiece(ByVal i As Integer) As ChessPiece
      Return CType(chessPieces(i), ChessPiece)
   End Function ' GetPiece

   ' handle NewGame menu option click
   Private Sub newGameItem_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles newGameItem.Click

      ResetBoard() ' reinitialize board
      Invalidate() ' refresh form
   End Sub ' newGameItem_Click 
End Class ' FrmChessGame

'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************
