' Fig. 18.11: FrmReadSequentialAccessFile.vb
' Reading a sequential-access file.
Imports System.IO
Imports BankLibrary

Public Class FrmReadSequentialAccessFile
   Private input As FileStream ' maintains connection to a file
   Private fileReader As StreamReader ' reads data from a text file

   ' invoked when user clicks the Open button
   Private Sub btnOpen_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnOpen.Click
      ' create dialog box enabling user to open file
      Dim fileChooser As New OpenFileDialog()
      Dim result As DialogResult = fileChooser.ShowDialog()
      Dim fileName As String ' name of file containing data

      ' exit event handler if user clicked Cancel
      If result = Windows.Forms.DialogResult.Cancel Then
         Return
      End If

      fileName = fileChooser.FileName ' get specified file name
      ClearTextBoxes()

      ' show error if user specified invalid file
      If fileName = "" Or fileName Is Nothing Then
         MessageBox.Show("Invalid File Name", "Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
      Else
         ' create FileStream to obtain read access to file
         input = New FileStream(fileName, FileMode.Open, FileAccess.Read)

         ' set file from where data is read
         fileReader = New StreamReader(input)

         btnOpen.Enabled = False ' disable Open File button
         btnNext.Enabled = True ' enable Next Record button
      End If
   End Sub ' btnOpen_Click

   ' invoked when user clicks Next button
   Private Sub btnNext_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles btnNext.Click

      Try
         ' get next record available in file
         Dim inputRecord As String = fileReader.ReadLine()

         ' will store individual pieces of data
         Dim inputFields() As String

         If inputRecord IsNot Nothing Then
            inputFields = inputRecord.Split(","c)

            Dim record As New Record(Convert.ToInt32( _
               inputFields(0)), inputFields(1), inputFields(2), _
               Convert.ToDecimal(inputFields(3)))

            ' copy string array values to TextBox values
            SetTextBoxValues(inputFields)
         Else
            fileReader.Close() ' close StreamReader
            input.Close() ' close FileStream if no Records in file
            btnOpen.Enabled = True ' enable Open File button
            btnNext.Enabled = False ' disable Next Record button
            ClearTextBoxes()

            ' notify user if no Records in file
            MessageBox.Show("No more records in file", "", _
               MessageBoxButtons.OK, MessageBoxIcon.Information)
         End If
      Catch ex As IOException
         MessageBox.Show("Error Reading from File", "Error", _
            MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
   End Sub ' btnNext_Click
End Class ' FrmReadSequentialAccessFile


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************