' Fig. 22.9: HugeInteger.vb
' HugeInteger Web service performs operations on large integers.
Imports System.Web
Imports System.Web.Services
Imports System.Web.Services.Protocols

<WebService(Namespace:="http://www.deitel.com/")> _
<WebServiceBinding(ConformsTo:=WsiProfiles.BasicProfile1_1)> _
<Global.Microsoft.VisualBasic.CompilerServices.DesignerGenerated()> _
Public Class HugeInteger
   Inherits System.Web.Services.WebService

   Private Const MAXIMUM As Integer = 100 ' maximum number of digits
   Public number As Integer() ' array representing the huge integer

   ' default constructor
   Public Sub New()
      number = New Integer(MAXIMUM - 1) {}
   End Sub ' New

   ' property that accepts an integer parameter
   Public Property Digit(ByVal index As Integer) As Integer
      Get
         Return number(index)
      End Get

      Set(ByVal Value As Integer)
         number(index) = Value
      End Set
   End Property ' Digit

   ' returns String representation of HugeInteger
   Public Overrides Function ToString() As String
      Dim returnString As String = ""

      For Each digit As Integer In number
         returnString = digit & returnString
      Next

      Return returnString
   End Function ' ToString

   ' creates HugeInteger based on argument
   Public Shared Function FromString(ByVal value As String) As HugeInteger
      ' create temporary HugeInteger to be returned by the method
      Dim parsedInteger As New HugeInteger()

      For i As Integer = 0 To value.Length - 1
         parsedInteger.Digit(i) = Int32.Parse( _
            value.Chars(value.Length - i - 1).ToString())
      Next

      Return parsedInteger
   End Function ' FromString

   ' WebMethod that adds integers represented by the String arguments
   <WebMethod(Description:="Adds two huge integers.")> _
   Public Function Add(ByVal first As String, ByVal second As String) _
      As String

      Dim carry As Integer = 0
      Dim operand1 As HugeInteger = HugeInteger.FromString(first)
      Dim operand2 As HugeInteger = HugeInteger.FromString(second)
      Dim result As New HugeInteger() ' stores result of addition

      ' perform addition algorithm for each digit
      For i As Integer = 0 To MAXIMUM - 1
         ' add two digits in same column
         ' result is their sum, plus carry from 
         ' previous operation modulo 10
         result.Digit(i) = _
            (operand1.Digit(i) + operand2.Digit(i) + carry) Mod 10

         ' set carry to remainder of dividing sums of two digits by 10
         carry = (operand1.Digit(i) + operand2.Digit(i) + carry) \ 10
      Next

      Return result.ToString()
   End Function ' Add

   ' WebMethod that subtracts integers represented by the String arguments
   <WebMethod(Description:="Subtracts two huge integers.")> _
   Public Function Subtract(ByVal first As String, _
      ByVal second As String) As String

      Dim operand1 As HugeInteger = HugeInteger.FromString(first)
      Dim operand2 As HugeInteger = HugeInteger.FromString(second)
      Dim result As New HugeInteger()

      ' subtract bottom digit from top digit
      For i As Integer = 0 To MAXIMUM - 1
         ' if top digit is smaller than bottom digit we need to borrow
         If operand1.Digit(i) < operand2.Digit(i) Then
            Borrow(operand1, i)
         End If

         ' subtract bottom from top
         result.Digit(i) = operand1.Digit(i) - operand2.Digit(i)
      Next

      Return result.ToString()
   End Function ' Subtract

   ' borrow 1 from next digit
   Private Sub Borrow(ByVal hugeInteger As HugeInteger, _
      ByVal place As Integer)

      ' if no place to borrow from, signal problem
      If place >= MAXIMUM - 1 Then
         Throw New ArgumentException()

         ' otherwise if next digit is zero, borrow from column to left
      ElseIf hugeInteger.Digit(place + 1) = 0 Then
         Borrow(hugeInteger, place + 1)
      End If

      ' add ten to current place because we borrowed and subtract one from
      ' previous digit--this is the digit we borrowed from
      hugeInteger.Digit(place) += 10
      hugeInteger.Digit(place + 1) -= 1
   End Sub ' Borrow

   ' WebMethod that returns true if first integer is bigger than second
   <WebMethod(Description:="Determines whether the first integer is " & _
      "larger than the second integer.")> _
   Public Function Bigger(ByVal first As String, _
      ByVal second As String) As Boolean

      Dim zeros() As Char = {"0"}

      Try
         ' if elimination of all zeros from result 
         ' of subtraction is an empty string,
         ' numbers are equal, so return false, otherwise return true
         If Subtract(first, second).Trim(zeros) = "" Then
            Return False
         Else
            Return True
         End If
      Catch exception As ArgumentException
         Return False ' first number was smaller, so return False
      End Try
   End Function ' Bigger

   ' WebMethod returns True if first integer is smaller than second
   <WebMethod(Description:="Determines whether the first integer " & _
      "is smaller than the second integer.")> _
   Public Function Smaller(ByVal first As String, _
      ByVal second As String) As Boolean
      ' if second is bigger than first, then first is smaller than second
      Return Bigger(second, first)
   End Function ' Smaller

   ' WebMethod that returns true if two integers are equal
   <WebMethod(Description:="Determines whether the first integer " & _
      "is equal to the second integer.")> _
   Public Function EqualTo(ByVal first As String, _
      ByVal second As String) As Boolean

      ' if either first is bigger than second, or first is 
      ' smaller than second, they are not equal
      If Bigger(first, second) OrElse Smaller(first, second) Then
         Return False
      Else
         Return True
      End If
   End Function ' EqualTo
End Class ' HugeInteger


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************
