' Fig. 23.4: FrmPacketClient.vb
' Set up a client that sends packets to a server and receives 
' packets from a server.
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading

Public Class FrmPacketClient
   Private client As UdpClient
   Private receivePoint As IPEndPoint

   ' initialize variables and thread for receiving packets
   Private Sub FrmPacketClient_Load(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles MyBase.Load

      receivePoint = New IPEndPoint(New IPAddress(0), 0)
      client = New UdpClient(50001)
      Dim thread As New Thread(New ThreadStart(AddressOf WaitForPackets))
      thread.Start()
   End Sub ' FrmPacketClient_Load

   ' shut down the client
   Private Sub FrmPacketClient_FormClosing( _
      ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.FormClosingEventArgs) _
      Handles MyBase.FormClosing

      System.Environment.Exit(System.Environment.ExitCode)
   End Sub ' FrmPacketClient_FormClosing

   ' Delegate that allows method DisplayMessage to be called
   ' in the thread that creates and maintains the GUI       
   Delegate Sub DisplayDelegate(ByVal message As String)

   ' method DisplayMessage sets txtDisplay's Text property
   ' in a thread-safe manner
   Private Sub DisplayMessage(ByVal message As String)
      ' if modifying txtDisplay is not thread safe
      If txtDisplay.InvokeRequired Then
         ' use inherited method Invoke to execute DisplayMessage
         ' via a Delegate                                       
         Invoke(New DisplayDelegate(AddressOf DisplayMessage), _
            New Object() {message})
         ' OK to modify txtDisplay in current thread
      Else
         txtDisplay.Text &= message
      End If
   End Sub ' DisplayMessage

   ' send a packet
   Private Sub txtInput_KeyDown(ByVal sender As System.Object, _
      ByVal e As System.Windows.Forms.KeyEventArgs) _
      Handles txtInput.KeyDown

      If e.KeyCode = Keys.Enter Then
         ' create packet (datagram) as string
         Dim packet As String = txtInput.Text
         txtDisplay.Text &= _
            vbCrLf & "Sending packet containing: " & packet

         ' convert packet to Byte array                             
         Dim data As Byte() = System.Text.Encoding.ASCII.GetBytes(packet)

         ' send packet to server on port 50000               
         client.Send(data, data.Length, "localhost", 50000)
         txtDisplay.Text &= vbCrLf & "Packet sent" & vbCrLf
         txtInput.Clear()
      End If
   End Sub ' txtInput_KeyDown

   ' wait for packets to arrive
   Public Sub WaitForPackets()
      While True
         ' receive Byte array from server                
         Dim data As Byte() = client.Receive(receivePoint)

         ' output packet data to TextBox
         DisplayMessage( _
            vbCrLf & "Packet received:" & vbCrLf & "Length: " & _
            data.Length & vbCrLf & "Contents: " & _
            System.Text.Encoding.ASCII.GetString(data) & vbCrLf)
      End While
   End Sub ' WaitForPackets
End Class ' FrmPacketClient


'**************************************************************************
'* (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
'* Pearson Education, Inc. All Rights Reserved.                           *
'*                                                                        *
'* DISCLAIMER: The authors and publisher of this book have used their     *
'* best efforts in preparing the book. These efforts include the          *
'* development, research, and testing of the theories and programs        *
'* to determine their effectiveness. The authors and publisher make       *
'* no warranty of any kind, expressed or implied, with regard to these    *
'* programs or to the documentation contained in these books. The authors *
'* and publisher shall not be liable in any event for incidental or       *
'* consequential damages in connection with, or arising out of, the       *
'* furnishing, performance, or use of these programs.                     *
'**************************************************************************